package starj;

import java.util.*;

public class StorageFactory {
    private int flag_count;
    private int counter_count;
    private int storage_space_count;
    private Collection children;

    public StorageFactory() {
        this.flag_count = 0;
        this.counter_count = 0;
        this.storage_space_count = 0;
        this.children = new ArrayList();
    }
    
    public Storage[] getChildren() {
        Storage[] rv = new Storage[this.children.size()];
        return (Storage[]) this.children.toArray(rv);
    }
    
    public int size() {
        return this.children.size();
    }

    public int registerFlag() {
        for (Iterator i = this.children.iterator(); i.hasNext(); ) {
            ((StorageImpl) i.next()).registerFlag();
        }

        return this.flag_count++;
    }

    public int registerCounter() {
        for (Iterator i = this.children.iterator(); i.hasNext(); ) {
            ((StorageImpl) i.next()).registerCounter();
        }

        return this.counter_count++;
    }

    public int registerStorageSpace() {
        for (Iterator i = this.children.iterator(); i.hasNext(); ) {
            ((StorageImpl) i.next()).registerStorageSpace();
        }

        return this.storage_space_count++;
    }

    public Storage newStorage() {
        Storage storage = new StorageImpl(this.flag_count, this.counter_count,
                this.storage_space_count);
        this.children.add(storage);
        return storage;
    }

    private class StorageImpl implements Storage {
        private boolean[] flags;
        private long[] counters;
        private Object[] storage_spaces;

        StorageImpl(int flag_count, int counter_count,
                int storage_space_count) {
            if (flag_count > 0) {
                this.flags = new boolean[flag_count];
            }

            if (counter_count > 0) {
                this.counters = new long[counter_count];
            }

            if (storage_space_count > 0) {
                this.storage_spaces = new Object[storage_space_count];
            }
        }

        private int registerFlag() {
            int new_id;
            if (this.flags == null) {
                 new_id = 0;
            } else {
                new_id = this.flags.length;
            }
            boolean[] new_flags = new boolean[new_id + 1];
            if (this.flags != null) {
                System.arraycopy(this.flags, 0, new_flags, 0, new_id);
            }
            this.flags = new_flags;
            return new_id;
        }

        public boolean getFlag(int storage_id) {
            return this.flags[storage_id];
        }

        public void setFlag(int storage_id, boolean value) {
            this.flags[storage_id] = value;
        }

        private int registerCounter() {
            int new_id;
            if (this.counters == null) {
                 new_id = 0;
            } else {
                new_id = this.counters.length;
            }
            long[] new_counters = new long[new_id + 1];
            if (this.counters != null) {
                System.arraycopy(this.counters, 0, new_counters, 0, new_id);
            }
            this.counters = new_counters;
            return new_id;
        }

        public long getCounter(int storage_id) {
            return this.counters[storage_id];
        }

        public void stepCounter(int storage_id) {
            this.counters[storage_id] += 1L;
        }

        public void decrCounter(int storage_id) {
            this.counters[storage_id] -= 1L;
        }

        public void addToCounter(int storage_id, long delta) {
            this.counters[storage_id] += delta;
        }

        public void setCounter(int storage_id, long value) {
            this.counters[storage_id] = value;
        }

        private int registerStorageSpace() {
            int new_id;
            if (this.storage_spaces == null) {
                 new_id = 0;
            } else {
                new_id = this.storage_spaces.length;
            }
            Object[] new_storage_spaces = new Object[new_id + 1];
            if (this.storage_spaces != null) {
                System.arraycopy(this.storage_spaces, 0, new_storage_spaces, 0,
                        new_id);
            }
            this.storage_spaces = new_storage_spaces;
            return new_id;
        }

        public Object getStorageSpace(int storage_id) {
            return this.storage_spaces[storage_id];
        }

        public void setStorageSpace(int storage_id, Object value) {
            this.storage_spaces[storage_id] = value;
        }
    }
}
